' Program: FEATURES.BAS (Demonstrate )
' This program demonstrates most of the major features of the 
' ILM-216 serial LCD module. It assumes that the ILM-216
' is set for 2400 bps (factory default) and that it is 
' connected to the Stamp as follows: 
'            ILM-216                 Stamp I/O
'            -----------------       ---------
'            pin 3 (serial in)         pin 0
'            pin 4 (serial out)        pin 1
' The program demonstrates the switch-input capability of
' the unit, so it will be more interesting if you actually
' have switches connected. If not, no harm done--the 
' program will just tell you that no switches are pressed!
' Likewise a piezo buzzer--if no buzzer is connected, the 
' 'bell' instruction will have no effect. 
' Note that only the first pause in the program is really 
' necessary ( to allow the LCD to initialize at power-on). 
' All other pauses are to slow the program down so that you 
' can see what's happening. 

' ===========Define names for LCD instructions==========
SYMBOL clrLCD = 12	' Clear entire LCD screen. 
SYMBOL posCmd = 16	' Position cursor. 
SYMBOL bell   = 7 	' Ring piezo-buzzer 'bell.'
SYMBOL liteON = 14	' Turn on the backlight. 
SYMBOL liteOFF= 15	' "    off "  "
SYMBOL ESC    = 27	' Escape code for special instructions. 
SYMBOL Ret    = 13	' Carriage return code. 

' =================Begin demonstration=================
pause 1000				' Wait a sec. 
serout 0,n2400,(clrLCD) 		' Clear the screen. 
serout 0,n2400,("ILM Feature Demo")	' Print title. 
pause 1000
serout 0,n2400,(bell, liteON)		' Ring bell, backlight on. 

' Print a countdown in the middle of the 2nd line. Character
' positions are numbered 0-31 with 0 at upper left and 31 at lower
' right. The 1-byte method of sending a position is to add 64
' and send the resulting byte. So position 24 (middle of line 2)
' is sent as 64 + 24 = 88. 
for b2 = 5 to 0 step -1			' Countdown from 5 to 0. 
  serout 0,n2400,(posCmd,88,#b2)	' Print value of b2 on line 2. 
  pause 1000				' wait a sec. 
next					' Repeat. 

' Clear the screen and poll the keys. Using ESC K0 causes the
' ILM-216 to send back a single byte whose lower 4 bits are
' the states of switches S1 through S4 (where 0 is open, 1 is closed). 
' The upper four bits of this byte are always %0100. The easiest way 
' to determine the state of a particular bit/switch is to store the 
' returned byte in variable b0. Remember that b0 can also be addressed
' as a series of bit variables, bit0 through bit7. Bit0 through 
' bit3 will contain the states of S1 through S4, respectively. 
' If you're comfortable with Boolean logic, you can also use the 
' AND function to look at particular bits, or you can use the returned
' byte as an input to a Branch or Lookup instruction. Before doing 
' so, make sure to strip off the upper four bits by ANDing the byte 
' with %00001111. 

serout 0,n2400,(clrLCD,liteOFF)		' Clear the LCD; backlight off.  
serout 0,n2400,("Keypress Demo")

readKeys:
  serout 0,n2400,(ESC,"K0")	' Have ILM-216 look at the keys. 
  serin 1,n2400,b0		' Get the returned byte in b0. 
  serout 0,n2400,(posCmd,86)	' Position cursor to mid line 2. 
  if bit0=1 then S1press
  if bit1=1 then S2press
  if bit2=1 then S3press
  if bit3=1 then S4press  
  serout 0,n2400,("--")
goto readKeys  
  
S1press:
  serout 0,n2400,("S1"): goto readKeys
S2press:
  serout 0,n2400,("S2"): goto readKeys
S3press:
  serout 0,n2400,("S3"): goto readKeys
S4press:
  serout 0,n2400,("S4"): goto readKeys
