DECLARE SUB textToBits ()
' Program: ILM.BAS
' The program is written to run from the DOS command line in the
' following format:
'
'              ILM COM1 FILENAME
'
' where COM1 is the device (COM1, COM2, DEC, or HEX) for output.
' Normally the output will be sent to a com port at 2400 bps
' to configure a connected ILM display. Output can also be directed
' to a text file with the specified number format (DEC or HEX). The
' file created this way will have the same name and path as the
' input filename, but with the file extension DEC or HEX.
' Filename is the DOS filename for a configuration text file.
' If the file is not in the current DOS path, the full path must
' be given.

' ===SUBS AND ARRAY DECLARARATIONS===
DEFINT A-Z
DIM SHARED configBytes(32) AS INTEGER
DIM SHARED lineToProcess$
DIM SHARED bitIndex AS INTEGER
bitIndex = 0
CLEAR
CLS

' ===INITIALIZATION AND COMMAND PARSING===
file$ = COMMAND$                ' Get the command-line argument.
' COMMAND$ strips any extra lefthand spaces and converts all input
' to uppercase. Extra spaces _between_ parameters are not removed.
space = INSTR(file$, " ")       ' Find 1st space character.
port$ = LEFT$(file$, space)     ' Up to 1st space is com port (1 or 2).
port$ = RTRIM$(port$)           ' Trim extra spaces.
theRest = LEN(file$) - space    ' Remove port spec from file$--everything
file$ = RIGHT$(file$, theRest)  ' from 1st space to end is now file$.

' Check for valid port selection and bail out with error message if bad.
SELECT CASE port$
  CASE "COM1", "COM2", "DEC", "HEX"
    ' No instruction here. Port is a valid choice.
  CASE ELSE
    GOTO badPort                ' Anything else invalid--squawk and quit.
END SELECT

dot = INSTR(file$, ".")         ' Find "." that separates name/extension.
IF dot = 0 THEN                 ' If no dot, then no extension given.
  file$ = file$ + ".TXT"        ' Default extension is "TXT"
END IF

' Get the six custom-character bit maps and pack them into the
' configBytes() array. The word "ASCII" marks the line immediately
' prior to the beginning of a set of bit-map data.
OPEN file$ FOR INPUT AS #1     ' Open the text file
FOR i = 1 TO 6
  DO UNTIL LEFT$(lineToProcess$, 5) = "ASCII"
    LINE INPUT #1, lineToProcess$
    lineToProcess$ = UCASE$(lineToProcess$)
  LOOP
  FOR j = 1 TO 8
    LINE INPUT #1, lineToProcess$
    textToBits
  NEXT j
NEXT i
PRINT "Custom characters configured."

' Now get the various configuration entries.
DO UNTIL UCASE$(LEFT$(lineToProcess$, 9)) = "DATA RATE"
  LINE INPUT #1, lineToProcess$
LOOP
PRINT lineToProcess$
i = (INSTR(lineToProcess$, "00") - 2)
cfgBaud$ = MID$(lineToProcess$, i, 2)
SELECT CASE cfgBaud$
  CASE "12"
   configBytes(30) = 3
  CASE "24"
   configBytes(30) = 2
  CASE "48"
   configBytes(30) = 1
  CASE "96"
   configBytes(30) = 0
  CASE ELSE
   configBytes(30) = 1          ' If bad, set for 2400.
END SELECT

DO UNTIL LEFT$(lineToProcess$, 9) = "BACKLIGHT"
  LINE INPUT #1, lineToProcess$
  lineToProcess$ = UCASE$(lineToProcess$)
LOOP
PRINT lineToProcess$
IF INSTR(lineToProcess$, "ON") <> 0 THEN
  configBytes(30) = configBytes(30) + 4
END IF

DO UNTIL LEFT$(lineToProcess$, 6) = "SCREEN"
  LINE INPUT #1, lineToProcess$
  lineToProcess$ = UCASE$(lineToProcess$)
LOOP
PRINT lineToProcess$
IF INSTR(lineToProcess$, "EEPROM") <> 0 THEN
  configBytes(30) = configBytes(30) + 8
END IF

DO UNTIL LEFT$(lineToProcess$, 5) = "DELAY"
  LINE INPUT #1, lineToProcess$
  lineToProcess$ = UCASE$(lineToProcess$)
LOOP
PRINT lineToProcess$
i = INSTR(lineToProcess$, "=")
i = LEN(lineToProcess$) - i
lineToProcess$ = RIGHT$(lineToProcess$, i)
configBytes(31) = VAL(lineToProcess$) AND 15

DO UNTIL LEFT$(lineToProcess$, 5) = "LINE1"
  LINE INPUT #1, lineToProcess$
LOOP
PRINT lineToProcess$
i = INSTR(lineToProcess$, "=")
i = LEN(lineToProcess$) - i
line1$ = LEFT$(RIGHT$(lineToProcess$, i) + SPACE$(16), 16)

DO UNTIL LEFT$(lineToProcess$, 5) = "LINE2"
  LINE INPUT #1, lineToProcess$
LOOP
PRINT lineToProcess$
i = INSTR(lineToProcess$, "=")
i = LEN(lineToProcess$) - i
line2$ = LEFT$(RIGHT$(lineToProcess$, i) + SPACE$(16), 16)
CLOSE   ' Close the file.

' Open the specified com port.
SELECT CASE port$
CASE "COM1", "COM2"
  OPEN port$ + ": 2400,N,8,1,CD0,CS0,DS0,OP0" FOR OUTPUT AS #1
CASE "DEC"
  start = LEN(file$) - 2
  MID$(file$, start) = "DEC"
  OPEN file$ FOR OUTPUT AS #1
CASE "HEX"
  start = LEN(file$) - 2
  MID$(file$, start) = "HEX"
  OPEN file$ FOR OUTPUT AS #1
END SELECT

FOR i = 0 TO 31
SELECT CASE port$
CASE "COM1", "COM2"
  PRINT #1, CHR$(configBytes(i));       ' Output config bytes

CASE "DEC"
  IF (i MOD 8) = 0 THEN
    PRINT #1, CHR$(13); CHR$(10); configBytes(i);
  ELSE
    PRINT #1, ","; configBytes(i);
  END IF
CASE "HEX"
  IF (i MOD 16) = 0 THEN
    PRINT #1, CHR$(13); CHR$(10); "$"; HEX$(configBytes(i));
  ELSE
    PRINT #1, ","; "$"; HEX$(configBytes(i));
  END IF
END SELECT
NEXT i

SELECT CASE port$
CASE "HEX", "DEC"
  PRINT #1, CHR$(13); CHR$(10);
  PRINT #1, CHR$(34); line1$; CHR$(34)  ' Now store the text.
  PRINT #1, CHR$(34); line2$; CHR$(34)
CASE "COM1", "COM2"
  PRINT #1, line1$;
  PRINT #1, line2$;
END SELECT
CLOSE                                  ' Close the port or file.
  

END

badPort:
  PRINT : PRINT "Configuration cancelled. Must be COM1, COM2, DEC, or HEX."
  END

SUB textToBits
i = LEN(lineToProcess$)
pixels = 1
DO WHILE i > 0 AND pixels < 6
  temp$ = MID$(lineToProcess$, i, 1)
  byteIndex = bitIndex \ 8
  theBit = 2 ^ (bitIndex MOD 8)
  SELECT CASE temp$
    CASE " "
    CASE "."
      bitIndex = bitIndex + 1
      pixels = pixels + 1
    CASE "!" TO "-", "/" TO "}"
      configBytes(byteIndex) = configBytes(byteIndex) OR theBit
      bitIndex = bitIndex + 1
      pixels = pixels + 1
  END SELECT
i = i - 1
LOOP
END SUB

